mod config;
mod modem;
#[allow(dead_code)]
mod command;

use anyhow;
use std::time::Duration;
use std::thread;
use esp_idf_hal::delay;
use esp_idf_hal::prelude::*;
use esp_idf_hal::peripherals::Peripherals;
use esp_idf_hal::serial;

use mqtt::control::ConnectReturnCode;
use mqtt::packet::{ConnackPacket, ConnectPacket, PublishPacketRef, QoSWithPacketIdentifier};
use mqtt::{Decodable, Encodable, TopicName};


fn main() -> anyhow::Result<()> {
    esp_idf_sys::link_patches();

    let dp = Peripherals::take().expect("error taking peripherals");

    // LilyGo TTGO T-Call sim800l board serial pins.
    let serial_rx = dp.pins.gpio26;
    let serial_tx = dp.pins.gpio27;

    let serial_pins = serial::Pins {
        tx: serial_tx,
        rx: serial_rx,
        cts: None,
        rts: None,
    };

    // Create the serial and panic with a message ... if we can't create the serial port, then we
    // can't communicate with the sim800l module, hence we don't run anymore.
    let serial: serial::Serial<serial::UART1, _, _> = serial::Serial::new(
        dp.uart1,
        serial_pins,
        serial::config::Config::default().baudrate(Hertz(115200)),
    )?;

    let (tx, rx) = serial.split();
    let mut mdm = modem::Modem::new(tx, rx);

    let modem_pwrkey = dp.pins.gpio4.into_output()?;
    let modem_rst = dp.pins.gpio5.into_output()?;
    let modem_power = dp.pins.gpio23.into_output()?;

    mdm.init(modem_pwrkey, modem_rst, modem_power)?;

    if !mdm.is_gprs_attached()? {
        let _ = mdm.gprs_attach_ap(
            config::A1_GPRS_AP.apn,
            config::A1_GPRS_AP.username,
            config::A1_GPRS_AP.password,
        )?;
    }

    thread::sleep(Duration::from_millis(2000));

    let _ = mdm.chip_info()?;

    loop {
        if mdm.is_gprs_attached()? {
            let _ = mdm.gprs_connect()?;
            for _ in 0..3 {
                let ip_addr = mdm.gprs_status()?;
                if ip_addr.contains("0.0.0.0") {
                    thread::sleep(Duration::from_millis(2000));
                } else {
                    break
                }
            }

            let _ = mdm.location()?;
            let _ = mdm.ssl_opt()?;

            println!("connecting to server!");
            //if !mdm.tcp_is_ssl_enabled()? {
            //    let _ = mdm.tcp_ssl_enable()?;
            //}
            //if mdm.tcp_is_ssl_enabled()? {
            //    let _ = mdm.tcp_ssl_disable()?;
            //}
            //
            let message = "{\"lat\": 20.475370, \"long\": 44.747224}";
            let url = "https://a.tracker.called.quest";
            //let url = "https://rest.iot.fr-par.scw.cloud";
            let token = "eyJOZXRJRCI6ImUwYTc5YzM2LWExZjYtNDNiMC1hMGY3LWE2YTk1OGI3Zjk1ZCIsIk5ldEtleSI6IjZhZGU2ZWZkLThiMGYtNDQ3ZC1hNWY5LThkNDJjNDk4NDQ3MSJ9";
            let reply = mdm.http_post(url, token, message.as_bytes())?;
            println!("+++++++++++++++++++++++++++++++++");
            println!("REPLY({}) = {}", reply.len(), reply);
            println!("+++++++++++++++++++++++++++++++++");
            let _ = mdm.http_close()?;

            break
        }
    }

    Ok(())
}
